#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jun 29 18:14:40 2020

@author: andrewthomson
"""

#%% IMPORT REQUIRED LIBRARIES
import pandas as pd
import numpy as np

from sklearn.preprocessing import StandardScaler
scaler = StandardScaler()

import pickle #for python 3.8
#import pickle5 as pickle #for python 3.7 or below

#%% LOAD RFR BAROMETER FILES

# SERIALISED RFR BAROMETER LOAD - change filename to match barometer calibration if required
pkl_filename = "pickle_model_20201210.pkl"

model_rf =  pickle.load(open(pkl_filename, "rb"))

# SERIALISED SCALER LOAD - change filename to match scaler calibration if required **MUST USE SCALER THAT GOES WITH BAROMETER**
scaler = pickle.load(open('scaler_20201210.pkl','rb'))

#%% LOAD DATA AND FILTER

data = pd.read_csv('example_test_data.txt',sep="\t")

predictors = ["Si", "Na", "Al", "Mg", "Fet", "Ca", "Mn", "Cr", "Ti"]

namj = np.empty([len(data)])
maj = np.empty([len(data)])
tot_maj = np.empty([len(data)])
added = np.empty([len(data)])
for i in range(0,len(data)):
    if (data.Na[i]-data.Ti[i])/2 >= 0:
        namj[i] = (data.Na[i]-data.Ti[i])/2
    else:
        namj[i] = 0

    if (data.Si[i]-(2*namj[i])-3) >= 0:
        maj[i] = (data.Si[i]-(2*namj[i])-3)
    else:
        maj[i] = 0
        
    tot_maj[i] = namj[i]+maj[i]
    
    added[i] = data.Si[i] + data.Ti[i] + data.Al[i] + data.Cr[i] + data.Fet[i] + data.Mn[i] + data.Mg[i] + data.Ca[i] + data.Na[i]
    

data['added'] = added
data['tot_maj'] = tot_maj
data2 = data[data['tot_maj'] >= 0.005]
data3 = data2[data2['added'] >= 7.9]
data4 = data3[data3['added'] <= 8.1]
    
del data2, data3, added, tot_maj, namj, maj
data4 = data4.reset_index(drop=True)

data_in = data4[predictors]

#%% APPLY BAROMETER OVER 100 ITERATIONS WITH RANDOM CHEMICAL UNCERTAINTIES

repeat = 100
# Create arrays that will be populated in RFR loop
nopredict = len(predictors)

preds_noise = np.empty([len(data_in),repeat])

for i in range(0,repeat): 
    test = data_in/data_in
    test['Si'] = test['Si'] + np.random.normal(0,1,size=test['Si'].shape) / 500
    test['Ti'] = test['Ti'] + np.random.normal(0,1,size=test['Ti'].shape) / 33
    test['Al'] = test['Al'] + np.random.normal(0,1,size=test['Al'].shape) / 100
    test['Cr'] = test['Cr'] + np.random.normal(0,1,size=test['Cr'].shape) / 20
    test['Fet'] = test['Fet'] + np.random.normal(0,1,size=test['Fet'].shape) / 33
    test['Mn'] = test['Mn'] + np.random.normal(0,1,size=test['Mn'].shape) / 10
    test['Mg'] = test['Mg'] + np.random.normal(0,1,size=test['Mg'].shape) / 33
    test['Ca'] = test['Ca'] + np.random.normal(0,1,size=test['Ca'].shape) / 100
    test['Na'] = test['Na'] + np.random.normal(0,1,size=test['Na'].shape) / 33
    test = test.fillna(1)
    data_temp = test * data_in
    data_temp[predictors] = scaler.transform(data_temp)
    data_in_temp = data_temp.values 

    preds_noise[:,i] = model_rf.predict(data_in_temp)
    
pressures_pred = np.empty([len(preds_noise)])
pre_std_pred = np.empty([len(preds_noise)])
for i in range(0,len(preds_noise)):
    pressures_pred[i] = np.mean(preds_noise[i,:])
    pre_std_pred[i] = 2 * np.std(preds_noise[i,:])

    #%%WRITE OUT PRESSURES FOR INPUT DATA
    
data4['Majorite_pressure'] = pressures_pred
data4['Pressure_error'] = pre_std_pred

data4.to_csv(r'out_pressures_test.txt',sep='\t',index=False,header=True)
